#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <gtk/gtk.h>

#include "param.h"
#include "util.h"
#include "pattern.h"
#include "layout.h"


static struct layout {
    const char *name;
    GdkPixmap *pixmap;
    struct layout *next;
} *layouts = NULL;


static GtkWidget *window;
static GtkWidget *image;


static gboolean button_press_event(GtkWidget *widget, GdkEventButton *event)
{
    if (event->button == 1)
	pattern_press(event->x-XSIZE/2, XSIZE/2-event->y);
    return TRUE;
}


static gboolean button_release_event(GtkWidget *widget, GdkEventButton *event)
{
    if (event->button == 1)
	pattern_release(event->x-XSIZE/2, XSIZE/2-event->y);
    return TRUE;
}


static gboolean motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
{
    assert(!event->is_hint);

    if (event->state & GDK_BUTTON1_MASK)
	pattern_track(event->x-XSIZE/2, XSIZE/2-event->y);
  
    return TRUE;
}


void layout_init(void)
{
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_name(window, "Input Wheel");

    image = gtk_image_new();
    gtk_widget_set_size_request(GTK_WIDGET(image), XSIZE, YSIZE);
    gtk_container_add(GTK_CONTAINER(window), image);
    gtk_widget_show(image);

    g_signal_connect(G_OBJECT(window), "motion_notify_event",
      G_CALLBACK(motion_notify_event), NULL);
    g_signal_connect(G_OBJECT(window), "button_press_event",
      G_CALLBACK(button_press_event), NULL);
    g_signal_connect(G_OBJECT(window), "button_release_event",
      G_CALLBACK(button_release_event), NULL);

    gtk_widget_set_events(window,
      GDK_LEAVE_NOTIFY_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
      GDK_POINTER_MOTION_MASK);

    gtk_widget_show(window);
}


void layout_store(const char *name, const uint8_t *xbm)
{
    struct layout *layout;
    GtkStyle *style;

    layout = alloc_type(struct layout);
    layout->name = name;
    style = gtk_widget_get_style(window);
    layout->pixmap = gdk_pixmap_create_from_data(window->window, (void *) xbm,
       XSIZE, YSIZE, -1, &style->black, &style->white);
    layout->next = layouts;
    layouts = layout;
}


static void show_layout(GdkPixmap *pixmap)
{
    gtk_image_set_from_pixmap(GTK_IMAGE(image), pixmap, NULL);
}


void layout_display(const char *name)
{
    struct layout *layout;

    for (layout = layouts; layout; layout = layout->next)
	if (!strcmp(layout->name, name)) {
	    show_layout(layout->pixmap);
	    return;
	}
    fprintf(stderr, "layout \"%s\" not found\n", name);
    exit(1);
	    
}
