#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>

#include "param.h"
#include "util.h"
#include "wheel.h"
#include "action.h"
#include "pattern.h"


#define debug 0


struct pattern {
    const struct action *action;
    struct pattern *next[MAX_ZONES];
} base, *curr;

static int last_zone;


void pattern_start(void)
{
    curr = &base;
}


static void provide(struct pattern **p)
{
    if (!*p) {
	*p = alloc_type(struct pattern);
	memset(*p, 0, sizeof(struct pattern));
    }
}


void pattern_enter(int zone)
{
    provide(curr->next+zone);
    curr = curr->next[zone];
}


void pattern_clear(void)
{
    /* @@@FIXME: the mother of all memory leaks ... */
    memset(&base, 0, sizeof(base));
}


const struct pattern *pattern_get(void)
{
    struct pattern *tmp;

    tmp = alloc_type(struct pattern);
    *tmp = base;
    return tmp;
}


static void recurse(struct pattern *to, const struct pattern *from)
{
    int i;

    if (from->action)
	to->action = from->action;
    for (i = 0; i != MAX_ZONES; i++)
	if (from->next[i]) {
	     provide(to->next+i);
	     recurse(to->next[i], from->next[i]);
	}
}


void pattern_merge(const struct pattern *p)
{
    recurse(&base, p);
}


void pattern_action(const struct action *action)
{
    curr->action = action;
}


void pattern_press(int x, int y)
{
    int zone;

    if (debug)
	fprintf(stderr, "PRESS %d %d %d\n", x, y, wheel_zone(x, y));
    zone = last_zone = wheel_zone(x, y);
    curr = zone == -1 ? NULL : base.next[zone];
}


void pattern_track(int x, int y)
{
    int zone;

    if (debug)
	fprintf(stderr, "TRACK %d %d %d\n", x, y, wheel_zone(x, y));
    if (!curr)
	return;
    zone = wheel_zone(x, y);
    if (zone == last_zone)
	return;
    last_zone = zone;
    curr = zone == -1 ? NULL : curr->next[zone];
}


void pattern_release(int x, int y)
{
    if (debug)
	fprintf(stderr, "RELEASE %d %d %d\n", x, y, wheel_zone(x, y));
    if (curr && curr->action)
	do_actions(curr->action);
    curr = NULL;
    gtk_main_quit();
}
